/* city - generates cities based on
 * "Medieval Demographics Made Easy" by S. John Ross 
 * http://www.io.com/~sjohn/demog.htm
 *
 * Reads in professional statistics from "profsv"
 * Generates cities based on those statistics
 *
 * Written by Ken Witherow <phantoml@frontiernet.net>
 *
 * This code is copylefted, see the included "COPYING" file for details
 * No warranty. Use at your own risk.
 * 
 */

#include "city.h"

#ifndef DEFAULTPROF
#define DEFAULTPROF "profsv" 
#endif

/* read in the profession file (if specified on the command line else 
 * default of ./profsv) storing the data in profession[]
 */
void readprofs( char *prof )
{
 int costprof = 1;
 char *InpLine, *buffer;
 FILE *datafile;
 char *proffile = DEFAULTPROF;  

 /* if a command line argument is specified, use it as the profession
  *file
  */
 
 if ( prof != NULL )
 	proffile = prof; 
 	
 buffer = malloc( 100 * sizeof( char ));

 if ( buffer == NULL )
 {
 	printf( "Out of memory.\n" );
 	exit(1);
 }
 
 /* ensure we've seeded the random numbers */
 srand( time(NULL) );
 numprofs = 0;
 
 /* Read in our data file and put it into profession
  * if leftmost character is % then ignore line
  * store total number of professions in numprofs
  */
 
 if ( ( datafile = fopen( proffile, "r" )) == NULL )
    datafile = fopen( "profsv", "r");

if ( datafile != NULL )
 {
 	 do
  	 {
   	    InpLine = fgets( buffer, 100, datafile ) ;
   	  	if ( InpLine[0] != '%' )
   		{
    		costprof = atoi( InpLine );
    		if ( costprof > 0 )
    		{
     			InpLine = fgets( buffer, 100, datafile );
     			if ( InpLine[0] != '%' )
     			{
      				numprofs++;
      				profession[numprofs].sv = costprof;
      				strncpy( profession[numprofs].name, InpLine, strlen(InpLine)-1 );
     			}
    		}
   		}
  	} while ( costprof > 0 );
    fclose( datafile );
 }
else
 {
 	fprintf( stderr, "Error opening file, %s!", proffile );
 	exit(1);
 }

}

/* divide the population by the cost of the profession
 * quot holds the number of units
 * for variety, check to see if an extra unit may be present
 * when it otherwise wouldn't
 */
int calcnum( long int costprof, long int population )
{
 int randomnum;
 ldiv_t good;
 good = ldiv( population, costprof );
 randomnum = rand() % costprof +1;

 /* if a random number is smaller number than the
  * remainder from our divide add an extra
  * 
  * eg, if a profession costs 200 people and there
  * are 150 left until the next plateau, if randnum
  * <= 150 then we'll add another professional
  */
 if ( !( randomnum > good.rem ) )
 	good.quot++;
 return ( good.quot );
}

/* store the number of units for each profession for a city
 *
 * if the profession has a '*' in it's title, limit it to a max of 1 for the city
 * if population is larger than 25000, it's guaranteed to have a *
 * 
 * if the profession has a '!' in it's title, force a minimum of 1 for the city
 */
void populatecity( long int population )
{
 int icounter = 0;
 do
 {
 	icounter++;
 	profession[icounter].calc = calcnum( profession[icounter].sv, population );
 	if ( strchr( profession[icounter].name, '*' ))
 	{
 		if ( profession[icounter].calc > 1 )
 			profession[icounter].calc = 1;
 		if ( population > 25000 )
			profession[icounter].calc = 1;
 	}
 	if ( strchr( profession[icounter].name, '!' ))
 	{
 		if ( profession[icounter].calc < 1 )
 			profession[icounter].calc = 1;
 	}
 } while ( icounter < numprofs );
}

/* print the current city out to the appropriate file */
void printcity( long int population, int domain )
{
 int icounter = 0;
 char *homedir = malloc(100); 
 FILE *datafile;

 strcpy( homedir, getenv( "HOME" ));
 
 if ( homedir != NULL )
    strcat( homedir, "/" );

 switch ( domain )
 {
 	case 0:
        strcat( homedir, "villages" );
 		break;
 	case 1:
 		strcat( homedir, "towns" );
 		break;
 	case 2:
        strcat( homedir, "cities" );
 		break;
	case 3:
		strcat( homedir, "single");
 }

 datafile = fopen( homedir, "a" );

 fprintf( datafile, "City: \n" );
 fprintf( datafile, "Population: %ld\n", population );
 totalprofession = 0;
 do
 {
 	icounter++;
	fprintf( datafile, "%u %s\n", profession[icounter].calc, profession[icounter].name );
    totalprofession += profession[icounter].calc;
 } while ( icounter < numprofs );
 fprintf( datafile, "Total number of professionals: %ld \n", totalprofession );
 fprintf( datafile, "Total number of non-professionals: %ld \n", population - totalprofession );
 fprintf( datafile, "\n" );
 fclose( datafile );
 totalprofession = 0;
 free( homedir );
}


