/* Console mode version of citygen
 *
 * Written by Ken Witherow <phantoml@frontiernet.net>
 *
 * This code is copylefted, see the included "COPYING" file for details
 *
 */

#include <stdlib.h>
#include <stdio.h>

/* contains city level routines */
#include "city.h" 
/* contains overall kingdom level routines */
#include "kingdom.h"

#if defined(__MSDOS__)
        #define SUBVERS "DOS"
#elif defined(_Windows)
        #define SUBVERS "WIN"
#else
        #define SUBVERS ""
#endif 

void printstats( void )
{
 printf( "%d Villages %d Towns %d Cities\n", getpopcenter(1), getpopcenter(2), getpopcenter(3) );
 printf( "%lu        %lu    %lu\n", getpop(1), getpop(2), getpop(3) );
 printf( " %lu Total population in the country living in centers of at least 20 people\n", getpop(1)+getpop(2)+getpop(3) );
 if ( villages == MAXARRAY )
	printf( "%s%d %s", "\nonly ", MAXARRAY, "of any one population center can be created. \n" );  		
}


int main( int argc, char *argv[] )
{
 /* population of kingdom from STDIN */
 unsigned long int population = 0; 	
 /* 0=village, 1=town, 2=city */
 int domaincount = 0;				
 /* age of kingdom */
 int age = 0;						
 /* size of current town */
 long int sizetown = 1;			
 /* used to determine if we're in single mode */
 int argcount = 0, singmode = 0;
 
 unsigned long int *popbuffer = malloc( sizeof( population ));
 int *agebuffer = malloc( sizeof( age ));

 if ( popbuffer == NULL || agebuffer == NULL )
 {
 	printf( "Out of memory.\n" );
 	exit(1);
 }	 
 
 printf( "Citygen %s%s\n", VERSION, SUBVERS );
 printf( "Copyright 199, Ken Witherow <phantoml@frontiernet.net>\n\n" );
 printf( "No warranty. Use at your own risk" );
 printf( "\nIf you wish to use a different professions file, use the following:\n" );

#if defined(__MSDOS__)
 printf( "citygen c:\\somedir\\myprofessions\n" );
#else
 printf( "citygen /somedirectory/hierarchy/myprofessions\n" );
#endif

 printf( "type -single on the command line to generate a single city\n" );
 /* Read in the proficiency file, using the command line if specified */
if ( argc > 0 )
		for( argcount = 0; argcount < argc; argcount++ )
			if ( strstr( argv[argcount], "-single" ) )
				{
					singmode = 1;
					break;
				} 
				
 if( singmode & (argcount == 1))
	readprofs( argv[2] );
 else
	readprofs( argv[1] );

 /* Get the population and age of kingdom */
 if ( singmode == 1 )
 	printf( "What is the population of the city: ");
 else
 	printf( "What is the population of the kingdom: ");
 scanf( "%lu", popbuffer );
 population = *popbuffer;
 printf( "How old (in years) is the kingdom: ");
 scanf( "%d", agebuffer );
 age = *agebuffer;

 /* check to see if we want to just generate one city */
 if ( singmode == 1 )
 {
 	populatecity( population );
 	printcity( population, 3 );
 	printf( "The output of the city generation is in the file 'single'\n" );
 	return(1);
 }


 /* make sure both are used */
 if ( ( population > 0 ) & ( age > 0 ) )
 {

	/* figure out number of villages, towns and cities and calculate population of each */
	calctowns( population, age );

	/* go through villages, towns and cities
	 * calculate amount for each profession
	 * print town to appropriate file 
	 */
	while ( domaincount < 3 )
	{
		while ( sizetown > 0 )
		{
			sizetown = sizeoftown( domaincount );
			if ( sizetown > 0 )
			{
				populatecity( sizetown );
				printcity( sizetown, domaincount );
			}
		};
		domaincount++;
		sizetown = 1;
	};
	/* dump calculated population stats to STDOUT */	
 	printstats();
 } 
 else
 	printf( "\n\nYou must enter a valid population and kingdom age\n" );
 return( 0 );
}
