/* GTK mode version of citygen
 *
 * Written by Ken Witherow <phantoml@frontiernet.net>
 *
 * This code is copylefted, see the included "COPYING" file for details
 * No warranty. Use at your own risk
 * 
 */

#include <stdlib.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include <ochregtk.h>

/* contains city level routines */
#include "city.h" 
/* contains overall kingdom level routines */
#include "kingdom.h"

GtkWidget *about;
GtkWidget *calcsingle;
GtkWidget *poptext;
GtkWidget *agetext;
GtkWidget *filew;

void gtkprintstats( void )
{
    GtkWidget *gtk_dialog_new( void );
    GtkWidget *finishedwindow;
    GtkWidget *finishedbutton;
    GtkWidget *finishedlabel;
    char *boxtext = malloc( 200 );
    
    finishedwindow = gtk_dialog_new();
    finishedbutton = gtk_button_new_with_label( "OK" );
    gtk_box_pack_start(GTK_BOX (GTK_DIALOG (finishedwindow)->action_area), finishedbutton, TRUE, TRUE, 0);
    sprintf( boxtext, "%d Villages %d Towns %d Cities", getpopcenter(1), getpopcenter(2), getpopcenter(3) );
    finishedlabel = gtk_label_new( boxtext );
    gtk_box_pack_start(GTK_BOX (GTK_DIALOG (finishedwindow)->vbox), finishedlabel, TRUE, TRUE, 0);
    sprintf( boxtext, "%lu        %lu    %lu\n", getpop(1), getpop(2), getpop(3) );
    finishedlabel = gtk_label_new( boxtext );
    gtk_box_pack_start(GTK_BOX (GTK_DIALOG (finishedwindow)->vbox), finishedlabel, TRUE, TRUE, 0);
    sprintf( boxtext, "  %lu Total population in the country living in centers of at least 20 people  \n", getpop(1)+getpop(2)+getpop(3) );
    finishedlabel = gtk_label_new( boxtext );
    gtk_box_pack_start(GTK_BOX (GTK_DIALOG (finishedwindow)->vbox), finishedlabel, TRUE, TRUE, 0);
    sprintf( boxtext, "  The results are in the files cities, towns and villages in your home directory  \n", getpop(1)+getpop(2)+getpop(3) );
    finishedlabel = gtk_label_new( boxtext );
    gtk_box_pack_start(GTK_BOX (GTK_DIALOG (finishedwindow)->vbox), finishedlabel, TRUE, TRUE, 0);
   
    gtk_signal_connect(GTK_OBJECT( finishedbutton ), "clicked", GTK_SIGNAL_FUNC( closepopup ), finishedwindow );

    gtk_widget_show_all( finishedwindow );    
}

void aboutpop( void )
{
    GtkWidget *gtk_dialog_new( void );
    GtkWidget *aboutwin;
    GtkWidget *aboutdone;
    GtkWidget *aboutlabel;
    char *aboutbox = malloc( 100 );

    aboutwin = gtk_dialog_new();
    aboutdone = gtk_button_new_with_label( "Done" );
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(aboutwin)->action_area), aboutdone, TRUE, TRUE, 0);
    sprintf( aboutbox, "Ochre CITY GENerator version %s", VERSION );
    aboutlabel = gtk_label_new( aboutbox );
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(aboutwin)->vbox), aboutlabel, TRUE, TRUE, 0);
    aboutlabel = gtk_label_new( "Copyright 1999, Ken Witherow. No warranty. Use at your own risk.\n\n" );
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(aboutwin)->vbox), aboutlabel, TRUE, TRUE, 0);
    aboutlabel = gtk_label_new( "This program creates kingdom and city demographics\n\n" );
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(aboutwin)->vbox), aboutlabel, TRUE, TRUE, 0);
    aboutlabel = gtk_label_new( " New versions of OCHRE can be found at http://www.frontiernet.net/~phantoml/adnd/ochre.html " );
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(aboutwin)->vbox), aboutlabel, TRUE, TRUE, 0);

    gtk_signal_connect(GTK_OBJECT( aboutdone ), "clicked", GTK_SIGNAL_FUNC( closepopup ), aboutwin);
    gtk_widget_show_all( aboutwin );
}

void docalc( GtkWidget *widget, GtkToggleButton *toggle ) 
{
    unsigned long int population = 0; /* kingdom population */
    int domaincount = 0; /* 0=village, 1=town, 2=city */
    int age = 0; /* kingdom's age */
    long int sizetown = 1; /* size of current town */
    int singmode = 0;
    char *filename;
    GtkWidget *gtk_dialog_new( void );
    GtkWidget *singlewin;
    GtkWidget *singlebutton;
    GtkWidget *singlelabel;

    population = strtoul( gtk_entry_get_text( poptext ), NULL, 10 );
    age = atoi( gtk_entry_get_text( agetext ));
    filename = gtk_file_selection_get_filename( filew );

    if ( ( population == 0 ) || ( age == 0 ) )
          return;
        
    if ( GTK_TOGGLE_BUTTON(toggle)->active )
        singmode = 1;    

    readprofs( filename );

    /* check to see if we want to just generate one city */
    if ( singmode == 1 )
    {
 	    populatecity( population );
 	    printcity( population, 3 );

 	    singlewin = gtk_dialog_new();
 	    singlebutton = gtk_button_new_with_label( "Ok" );
 	    gtk_box_pack_start(GTK_BOX(GTK_DIALOG (singlewin)->action_area), singlebutton, TRUE, TRUE, 0);
 	    singlelabel = gtk_label_new( " The city has been generated ");
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(singlewin)->vbox), singlelabel, TRUE, TRUE, 0);
 	    singlelabel = gtk_label_new("Output is in the file single in your home directory\n");
 	    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(singlewin)->vbox), singlelabel, TRUE, TRUE, 0);

 	    gtk_signal_connect(GTK_OBJECT( singlebutton ), "clicked", GTK_SIGNAL_FUNC( closepopup ), singlewin);

 	    gtk_widget_show_all( singlewin );
 	    return;
    }
    else
    {
        calctowns( population, age );
        while ( domaincount < 3 )
	    {
		    while ( sizetown > 0 )          
		    {
			     sizetown = sizeoftown( domaincount );
			     if ( sizetown > 0 )
			     {
				    populatecity( sizetown );
				    printcity( sizetown, domaincount );
			     }
		     };
		     domaincount++;
		     sizetown = 1;
	    };
    gtkprintstats();
    } 
}    

int main( int argc, char *argv[] )
{
    GtkWidget *window;
    GtkWidget *buttonquit;
    GtkWidget *buttonabout;
    GtkWidget *buttoncalc;
    GtkWidget *vboxmain;
    GtkWidget *hboxinps;
    GtkWidget *hboxlabels;
    GtkWidget *poplabel;
    GtkWidget *agelabel;
    GtkWidget *hboxbuttons;
    GtkWidget *filebutton;
        
    gtk_init(&argc, &argv);

    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW( window ), "Ochre CITYGEN");
    filew = gtk_file_selection_new( "Professions file" );
    gtk_file_selection_set_filename( filew, DEFAULTPROF );

    /* Setup about box */

  
    buttoncalc = gtk_button_new_with_label( "Calculate" );
    buttonquit = gtk_button_new_with_label( "Quit" );
    buttonabout = gtk_button_new_with_label( "About" );
    poplabel = gtk_label_new( "population" );
    agelabel = gtk_label_new( "kingdom age" );
    calcsingle = gtk_check_button_new_with_label( "Compute only one city instead of kingdom" );    
    poptext = gtk_entry_new_with_max_length( 10 );
    agetext = gtk_entry_new_with_max_length( 5 );
    filebutton = gtk_button_new_with_label( "Choose alternate profession file" );

    gtk_entry_set_editable(GTK_ENTRY( poptext ), TRUE );
    gtk_entry_set_editable(GTK_ENTRY( agetext ), TRUE );

  	/* connect the signal handlers */ 
    gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC( destroy ), NULL);
    gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC( delete_event ), NULL);
	gtk_signal_connect(GTK_OBJECT( buttonquit ), "clicked", GTK_SIGNAL_FUNC( destroy ), NULL);
    gtk_signal_connect(GTK_OBJECT( buttonabout ), "clicked", GTK_SIGNAL_FUNC( aboutpop ), buttonabout );
    gtk_signal_connect(GTK_OBJECT( buttoncalc ), "clicked", GTK_SIGNAL_FUNC( docalc ), calcsingle );
    gtk_signal_connect(GTK_OBJECT( filebutton ), "clicked", GTK_SIGNAL_FUNC( openpopup ), filew );
    gtk_signal_connect(GTK_OBJECT( GTK_FILE_SELECTION(filew)->ok_button ), "clicked", GTK_SIGNAL_FUNC( closepopup ), filew );;
    gtk_signal_connect(GTK_OBJECT( GTK_FILE_SELECTION(filew)->cancel_button ), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT( filew ));

           

    hboxinps = gtk_hbox_new( TRUE, 0 );
    gtk_box_pack_start(GTK_BOX( hboxinps ), poptext, FALSE, TRUE, 5);
    gtk_box_pack_start(GTK_BOX( hboxinps ), agetext, FALSE, TRUE, 5);

    hboxlabels = gtk_hbox_new( TRUE, 0 );
    gtk_box_pack_start(GTK_BOX( hboxlabels ), poplabel, FALSE, TRUE, 5);
    gtk_box_pack_start(GTK_BOX( hboxlabels ), agelabel, FALSE, TRUE, 5);

    hboxbuttons = gtk_hbox_new( TRUE, 0 );
    gtk_box_pack_start(GTK_BOX( hboxbuttons ), buttoncalc, FALSE, TRUE, 5);
    gtk_box_pack_start(GTK_BOX( hboxbuttons ), buttonabout, FALSE, TRUE, 5);
    gtk_box_pack_start(GTK_BOX( hboxbuttons ), buttonquit, FALSE, TRUE, 5);

    vboxmain = gtk_vbox_new( TRUE, 0 );
    gtk_box_pack_start(GTK_BOX( vboxmain ), hboxlabels, FALSE, TRUE, 5);
    gtk_box_pack_start(GTK_BOX( vboxmain ), hboxinps, FALSE, TRUE, 5);
    gtk_box_pack_start(GTK_BOX( vboxmain ), calcsingle, FALSE, TRUE, 5);
    gtk_box_pack_start(GTK_BOX( vboxmain ), filebutton, FALSE, TRUE, 5);
    gtk_box_pack_start(GTK_BOX( vboxmain ), hboxbuttons, FALSE, TRUE, 5);

    gtk_container_add(GTK_CONTAINER( window ), vboxmain );

    gtk_widget_show_all( window );
    gtk_main ();
          
    return(0);
 }
