/* Determines number of cities/towns/villages and their populations
 * based on "Medieval Demographics Made Easy" by S. John Ross 
 * http://www.io.com/~sjohn/demog.htm
 *
 * Written by Ken Witherow <phantoml@frontiernet.net>
 *
 * This code is copylefted, see the included "COPYING" file for details
 * No warranty. Use at your own risk.
 *
 */

#include "kingdom.h"

/* calculates estimated overall populations in v, t, and c
 * figures out the number of villages and towns based on that
 * with a maximum of maxarray (who really needs more than that? :) )
 * finally, figure out each v, t, c's individual populations )
 */
void calctowns( unsigned long int population, int age )
{
 /* ensure we've seeded the random number */
 srand( time(NULL) );

 countp[0] = 0;
 countp[1] = 0;
 countp[2] = 0;

 kingage = age;
 kingpop = population;
 
 popv = (.89 * kingpop);
 villages = popv / 700;
 popt = (.06 * kingpop);
 towns = popt / 5000;
 popc = (.03 * kingpop);
 if (villages > MAXARRAY ) villages = MAXARRAY;
 if (towns > MAXARRAY ) towns = MAXARRAY;
 populate();
} 

/* returns the size of the current v/t/c
 * v=0, t=1, c=2 and increments the pointer to the next v/t/c
 */
long int sizeoftown( int typeofdomain )
{
 countp[typeofdomain]++;
 return( pop[typeofdomain][countp[typeofdomain]] );
}

/* calculate the individual population sizes for v/t/c */
void populate( )
{
 populatevillages();
 populatetowns();
 populatecities();
}

/* calculate individual village size (20-1000 people) */
void populatevillages( )
{
 int icounter;
 popv = 0;
 for (icounter = 0; icounter < villages; icounter++ )
 {
 	pop[0][icounter] = rand( ) % 980 + 21;
	popv = popv + pop[0][icounter];
  };
}

/* calculate individual town size (1000-8000 people ) */
void populatetowns( )
{
 int icounter;
 popt = 0;
 for (icounter = 0; icounter < towns; icounter++ )
 {
 	pop[1][icounter] = rand( ) % 7000 + 1001;
	popt = popt + pop[1][icounter];
 };
}

/* determine the number of cities and their individual sizes
 * the biggest city is 12-18 * the sqrt of the kingdom's population
 * the next cities are 80-90% of the previous city's size
 * when the city size reaches 12000, the remaining total city
 * pop is divided by 10000 to determine the number of remaining cities
 * which are each populated with 10000-12000 people )
 */
void populatecities( int age )
{
 unsigned long int citycount = 0;
 if ( popc > 8000 )
 {
 	cities = 1;
 	pop[2][1] = sqrt( kingpop ) * (rand() % 8 + 11);
 	citycount = pop[2][1];
 	if ( citycount < popc )
 	{
    	cities++;
 		pop[2][2] = (rand( ) % 50 + 26) * pop[2][1] / 100;
 		citycount += pop[2][2];
 		do
 		{
 			pop[2][cities+1] = (rand() % 10 + 81) * pop[2][cities] / 100;
 			cities++;
			citycount += pop[2][cities];
	 	} while (pop[2][cities]> 12000);
		while ( citycount < popc )
		{
			cities++;
			pop[2][cities] = (((rand() % 4000) + 8001));
			citycount += pop[2][cities];
		};
	}
 } 
 popc = citycount;
}

extern long int getpop( int poptype )
{
 if (poptype == 1)
    return( popv );
 if (poptype == 2)
    return( popt );
 return( popc );
}

extern int getpopcenter( int popcent )
{
 if (popcent == 1)
    return( villages );
 if (popcent == 2)
    return( towns );
 return( cities );
}